package com.hz.pm.api.external;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.hz.pm.api.common.util.EnvironmentUtil;
import com.hz.pm.api.external.model.dto.*;
import com.hz.pm.api.external.model.enumeration.MhDictType;
import com.ningdatech.basic.exception.BizException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

import static cn.hutool.core.date.DatePattern.NORM_DATETIME_FORMATTER;

/**
 * <p>
 * MhApiClient
 * </p>
 *
 * @author WendyYang
 * @since 20:41 2023/12/20
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class MhApiClient {

    private final EnvironmentUtil environmentUtil;

    @Value("${mh.api-host:}")
    private String mhApiHost;

    @Value("${mh.zwdd-work-notice.client-id:}")
    private String zwddWorkNoticeClientId;

    @Value("${mh.zwdd-work-notice.client-secret:}")
    private String zwddWorkNoticeClientSecret;

    private static final String UNIT_URL = "/sync/unit";
    private static final String USER_URL = "/sync/user";
    private static final String EXPERT_URL = "/sync/expert";
    private static final String COMPANY_URL = "/sync/company";
    private static final String PROJECT_URL = "/project_info/getProjectInfoList";

    private static final String ZWDD_MSG_URL = "/message/send";

    private static final String DICT_URL = "/dict_data/listByType";


    public MhRetDTO<List<MhUnitDTO>> queryUnits() {
        String requestUrl = mhApiHost + UNIT_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhUnitDTO>>>() {
        }, false);
    }

    public MhRetDTO<List<MhCompanyDTO>> queryCompany() {
        String requestUrl = mhApiHost + COMPANY_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhCompanyDTO>>>() {
        }, false);
    }

    public MhRetDTO<List<MhUserDTO>> queryUsers(LocalDateTime syncDateTime) {
        String requestUrl = mhApiHost + USER_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl += "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhUserDTO>>>() {
        }, false);
    }

    public MhRetDTO<MhExpertDTO> queryExperts(LocalDateTime syncDateTime) {
        String requestUrl = mhApiHost + EXPERT_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl += "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<MhExpertDTO>>() {
        }, false);
    }

    public MhRetDTO<List<MhProjectDTO>> listProjects() {
        String requestUrl = mhApiHost + PROJECT_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhProjectDTO>>>() {
        }, false);
    }

    public boolean sendZwddWorkNotice(MhZwddWorkNoticeDTO notice) {
        notice.setClientId(zwddWorkNoticeClientId);
        notice.setClientSecret(zwddWorkNoticeClientSecret);
        String retBody = HttpUtil.post(mhApiHost + ZWDD_MSG_URL, JSONUtil.toJsonStr(notice));
        JSONObject retObj = JSONUtil.parseObj(retBody);
        if (retObj.containsKey("code") && retObj.getInt("code") == 200) {
            return Boolean.TRUE;
        }
        log.warn("发送浙政钉工作通知失败：{}", retObj);
        return Boolean.FALSE;
    }

    static final String DICT_RET_JSON = "{\n" +
            "    \"code\": 200,\n" +
            "    \"msg\": \"success\",\n" +
            "    \"data\": [\n" +
            "        {\n" +
            "            \"id\": \"035931f1fdb698cdf0d4cb7a04154a34\",\n" +
            "            \"dictSort\": 0,\n" +
            "            \"dictLabel\": \"竞争性切磋\",\n" +
            "            \"dictValue\": \"JZXQC\",\n" +
            "            \"dictType\": \"buy_way\",\n" +
            "            \"dictId\": \"6f578f6f717a5d5bb0bf63a9b0870239\",\n" +
            "            \"isDefault\": \"N\",\n" +
            "            \"status\": \"0\",\n" +
            "            \"remark\": null,\n" +
            "            \"createTime\": \"2024-01-3021:21:40\",\n" +
            "            \"updateTime\": null,\n" +
            "            \"createBy\": \"7b40b2e2d396930d4f8567fd3d47a39e\",\n" +
            "            \"updateBy\": \"\"\n" +
            "        },\n" +
            "        {\n" +
            "            \"id\": \"1b66784a1af14660186d5e3e40dec96b\",\n" +
            "            \"dictSort\": 0,\n" +
            "            \"dictLabel\": \"询价\",\n" +
            "            \"dictValue\": \"XJ\",\n" +
            "            \"dictType\": \"buy_way\",\n" +
            "            \"dictId\": \"6f578f6f717a5d5bb0bf63a9b0870239\",\n" +
            "            \"isDefault\": \"N\",\n" +
            "            \"status\": \"0\",\n" +
            "            \"remark\": null,\n" +
            "            \"createTime\": \"2024-01-3021:22:04\",\n" +
            "            \"updateTime\": null,\n" +
            "            \"createBy\": \"7b40b2e2d396930d4f8567fd3d47a39e\",\n" +
            "            \"updateBy\": \"\"\n" +
            "        },\n" +
            "        {\n" +
            "            \"id\": \"fc5d7c2cb97b15db54e3f6265df9e5f1\",\n" +
            "            \"dictSort\": 0,\n" +
            "            \"dictLabel\": \"邀请招标\",\n" +
            "            \"dictValue\": \"YQZB\",\n" +
            "            \"dictType\": \"buy_way\",\n" +
            "            \"dictId\": \"6f578f6f717a5d5bb0bf63a9b0870239\",\n" +
            "            \"isDefault\": \"N\",\n" +
            "            \"status\": \"0\",\n" +
            "            \"remark\": null,\n" +
            "            \"createTime\": \"2024-01-3021:18:39\",\n" +
            "            \"updateTime\": null,\n" +
            "            \"createBy\": \"7b40b2e2d396930d4f8567fd3d47a39e\",\n" +
            "            \"updateBy\": \"\"\n" +
            "        }\n" +
            "    ]\n" +
            "}";

    public List<MhDictDTO> dictListByType(MhDictType dictType) {
        String retBody;
        if (!environmentUtil.isDevEnv()) {
            String url = mhApiHost + DICT_URL;
            retBody = HttpUtil.get(url + "?dictType=" + dictType);
        } else {
            retBody = DICT_RET_JSON;
        }
        MhRetDTO<List<MhDictDTO>> retObj = JSONUtil.toBean(retBody,
                new TypeReference<MhRetDTO<List<MhDictDTO>>>() {
                }, false);
        if (retObj.isOk()) {
            return retObj.getData();
        } else {
            log.error("获取字典信息失败：{} {}", dictType, retBody);
            throw BizException.wrap("获取字典【%s】失败", dictType.getVal());
        }
    }

}
